using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using SimpleJSON;
using System.Threading;
using MVR;
using MeshVR;
using System.Linq;


namespace Stopper {
	public class LightOptions : MVRScript {

        bool isSessionPlugin = false;

        AdjustLightV2 adjustLight;

        List<Light> lights = new List<Light>();

        JSONStorableFloat customShadowResolution;
        JSONStorableFloat shadowBias;

        bool needUpdate = false;
        float defaultShadowBias = 0.014f;
		public override void Init() {
            isSessionPlugin = containingAtom.name == "CoreControl";         

            if(isSessionPlugin){
                SuperController.singleton.onAtomAddedHandlers += OnAtomAdded;
                SuperController.singleton.onAtomRemovedHandlers += OnAtomRemoved;
                foreach(Atom atom in SuperController.singleton.GetAtoms()){
                    OnAtomAdded(atom);
                }   
            } else {
                OnAtomAdded(containingAtom);
            }
           
            CreateTextField(new JSONStorableString("shadowResolution", "Note: Higher values can cause severe FPS drops. 0 defaults to standard menu setting. VeryHigh shadow quality appears to correspond to 12."), false).height = 10;
            customShadowResolution = AddSlider("Custom Shadow Resolution", 0f,0f, 16, (float val) => {
                int resolution = val == 0f? 0 : 1 << (int)val - 1;
                foreach(Light light in lights ){
                    light.shadowCustomResolution = resolution;
                }
            }, true);

            shadowBias = AddSlider("Shadow Bias", defaultShadowBias, 0f, 5f, (float val) => {
                foreach(Light light in lights ){
                    light.shadowBias = val;
                }
            });
		}

        public JSONStorableFloat AddSlider(string name, float defaultVal, float min, float max, JSONStorableFloat.SetFloatCallback onChange = null, bool wholeNumbers = false, bool rightSide = false){
            JSONStorableFloat storableFloat = new JSONStorableFloat(name, defaultVal, min, max, true, true);
            storableFloat.storeType = JSONStorableParam.StoreType.Full;
            UIDynamicSlider uiSlider = this.CreateSlider(storableFloat, rightSide);
            uiSlider.rangeAdjustEnabled = true;
            uiSlider.slider.wholeNumbers = wholeNumbers;
            RegisterFloat(storableFloat);
            
            if(onChange != null){
                storableFloat.setCallbackFunction = onChange;
            }
            
            return storableFloat;
	    }

        private void OnAtomAdded(Atom atom) {
            if(atom.type == "InvisibleLight"){
                adjustLight = atom.GetStorableByID("Light") as AdjustLightV2;
                if(adjustLight == null){
                    return;
                }
                lights.Add(adjustLight.GetComponent<Light>());
                needUpdate = true;
            }
        }

         private void OnAtomRemoved(Atom atom) {
            if(atom.type == "InvisibleLight"){
                adjustLight = atom.GetStorableByID("Light") as AdjustLightV2;
                if(adjustLight == null){
                    return;
                }
                lights.Remove(adjustLight.GetComponent<Light>());
            }
        }
		
		private void Start(){
			
		}
		
		private void Update(){
            if(needUpdate){
                needUpdate = false;
                customShadowResolution.setCallbackFunction(customShadowResolution.val);
                shadowBias.setCallbackFunction(shadowBias.val);
            }
		}
	
        private void OnEnable(){
            customShadowResolution.SetVal(customShadowResolution.val);
            shadowBias.SetVal(shadowBias.val);
            needUpdate = true;
        }
			
		private void OnDisable(){
            foreach(Light light in lights){
                light.shadowCustomResolution = 0;
                light.shadowBias = defaultShadowBias;
            }
		}
		
		private void OnDestroy(){
            SuperController.singleton.onAtomAddedHandlers -= OnAtomAdded;
            SuperController.singleton.onAtomRemovedHandlers -= OnAtomRemoved;
		}
    }
}
